@@ -1,6 +1,7 @@
 import numpy as np
 import numba as nb
 import vtk
+import time
 from .sphere_proximity import *
 
 @nb.jit(nopython=True)
@@ -13,8 +14,50 @@ def separating_axis(position,plane,U1,V1,W1,U2,V2,W2,U1_scale,
                                              np.abs(np.dot(V2*V2_scale,plane))+
                                              np.abs(np.dot(W2*W2_scale,plane)))
 
-@nb.jit(nopython=True)
+#@nb.jit(nopython=True)
 def obb(data,edge):
+    #C2 = (edge[0:3] + edge[3:6]) / 2
+    #C1 = (data[:,0:3] + data[:,3:6]) / 2
+    """
+    start = time.time()
+    collision = False
+    start = time.time()
+    cyl_2 = vtk.vtkTubeFilter()
+    line2 = vtk.vtkLineSource()
+    line2.SetPoint1(edge[0],edge[1],edge[2])
+    line2.SetPoint2(edge[3],edge[4],edge[5])
+    cyl_2.SetInputConnection(line2.GetOutputPort())
+    cyl_2.SetRadius(edge[21])
+    cyl_2.SetNumberOfSides(20)
+    cyl_2.CappingOn()
+    for i in range(data.shape[0]):
+        cyl_1 = vtk.vtkTubeFilter()
+        line1 = vtk.vtkLineSource()
+        line1.SetPoint1(data[i,0],data[i,1],data[i,2])
+        line1.SetPoint2(data[i,3],data[i,4],data[i,5])
+        cyl_1.SetInputConnection(line1.GetOutputPort())
+        cyl_1.SetRadius(data[i,21])
+        cyl_1.SetNumberOfSides(20)
+        cyl_1.CappingOn()
+        collision = vtk.vtkCollisionDetectionFilter()
+        collision.SetInputConnection(0,cyl_1.GetOutputPort())
+        collision.SetInputConnection(1,cyl_2.GetOutputPort())
+        transform = vtk.vtkTransform()
+        matrix = vtk.vtkMatrix4x4()
+        collision.SetTransform(0,transform)
+        collision.SetMatrix(1,matrix)
+        collision.SetCollisionModeToAllContacts()
+        collision.GenerateScalarsOn()
+        collision.Update()
+        if collision.GetNumberOfContacts() > 0:
+            collision = True
+            break
+    #print('OBB: {}'.format(time.time()-start))
+    end = time.time() - start
+    #print(end)
+    return collision
+    """
+
     C1 = (edge[0:3] + edge[3:6]) / 2
     C2 = (data[:,0:3] + data[:,3:6]) / 2
     Position = C2 - C1
@@ -36,37 +79,37 @@ def obb(data,edge):
     result = []
     for i in range(data.shape[0]):
         # U1 axis
-        if separating_axis(Position[i,:],edge[6:9],edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],edge[6:9],edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # V1 axis
-        if separating_axis(Position[i,:],edge[9:12],edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],edge[9:12],edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # W1 axis
-        if separating_axis(Position[i,:],edge[12:15],edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],edge[12:15],edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # U2 axis
-        if separating_axis(Position[i,:],data[i,6:9],edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],data[i,6:9],edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # V2 axis
-        if separating_axis(Position[i,:],data[i,9:12],edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],data[i,9:12],edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # W2 axis
-        if separating_axis(Position[i,:],data[i,12:15],edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],data[i,12:15],edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
@@ -74,69 +117,116 @@ def obb(data,edge):
         # Mixed Plane Separation Tests
         # U1 X U2
         plane = np.cross(edge[6:9],data[i,6:9])
-        if separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # U1 X V2
         plane = np.cross(edge[6:9],data[i,9:12])
-        if separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # U1 X W2
         plane = np.cross(edge[6:9],data[i,12:15])
-        if separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # V1 X U2
         plane = np.cross(edge[9:12],data[i,6:9])
-        if separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # V1 X V2
         plane = np.cross(edge[9:12],data[i,9:12])
-        if separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # V1 X W2
         plane = np.cross(edge[9:12],data[i,12:15])
-        if separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # W1 X U2
         plane = np.cross(edge[12:15],data[i,6:9])
-        if separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # W1 X V2
         plane = np.cross(edge[12:15],data[i,9:12])
-        if separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         # W1 X W2
         plane = np.cross(edge[12:15],data[i,12:15])
-        if separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
+        if not separating_axis(Position[i,:],plane,edge[6:9],edge[9:12],edge[12:15],
                            data[i,6:9],data[i,9:12],data[i,12:15],edge[21],edge[21],
                            edge[20]/2,data[i,21],data[i,21],data[i,20]/2):
             result.append(False)
             continue
         else:
+            """
+            colors = vtk.vtkNamedColors()
+            background_color = 'white'
+            cyl = vtk.vtkTubeFilter()
+            line = vtk.vtkLineSource()
+            line.SetPoint1(edge[0],edge[1],edge[2])
+            line.SetPoint2(edge[3],edge[4],edge[5])
+            cyl.SetInputConnection(line.GetOutputPort())
+            cyl.SetRadius(edge[21])
+            cyl.SetNumberOfSides(100)
+            #models.append(cyl)
+            mapper = vtk.vtkPolyDataMapper()
+            actor1  = vtk.vtkActor()
+            mapper.SetInputConnection(cyl.GetOutputPort())
+            actor1.SetMapper(mapper)
+            actor1.GetProperty().SetColor(colors.GetColor3d('green'))
+            #actors.append(actor)
+            colors = vtk.vtkNamedColors()
+            cyl = vtk.vtkTubeFilter()
+            line = vtk.vtkLineSource()
+            line.SetPoint1(data[i,0],data[i,1],data[i,2])
+            line.SetPoint2(data[i,3],data[i,4],data[i,5])
+            cyl.SetInputConnection(line.GetOutputPort())
+            cyl.SetRadius(data[i,21])
+            cyl.SetNumberOfSides(100)
+            #models.append(cyl)
+            mapper = vtk.vtkPolyDataMapper()
+            actor2  = vtk.vtkActor()
+            mapper.SetInputConnection(cyl.GetOutputPort())
+            actor2.SetMapper(mapper)
+            actor2.GetProperty().SetColor(colors.GetColor3d('green'))
+            #actors.append(actor)
+            renderer = vtk.vtkRenderer()
+            renderer.SetBackground(colors.GetColor3d(background_color))
+
+            render_window = vtk.vtkRenderWindow()
+            render_window.AddRenderer(renderer)
+            render_window.SetWindowName('SimVascular Vessel Collision')
+
+            interactor = vtk.vtkRenderWindowInteractor()
+            interactor.SetRenderWindow(render_window)
+            renderer.AddActor(actor1)
+            renderer.AddActor(actor2)
+            render_window.Render()
+            interactor.Start()
+            """
             return True
+
     return False
 
 def collision_free(data,results,idx,terminal,vessel,radius_buffer):
@@ -146,6 +236,7 @@ def collision_free(data,results,idx,terminal,vessel,radius_buffer):
     new_vessels = np.ones((3,data.shape[1]))*-1
     new_vessels[0,0:3] = data[vessel,0:3]
     new_vessels[0,3:6] = results[5][idx]
+    new_vessels[0,-1] = data[vessel,-1]
     if vessel == 0:
         new_vessels[0,21]  = results[0][idx] + radius_buffer
     else:
@@ -154,25 +245,49 @@ def collision_free(data,results,idx,terminal,vessel,radius_buffer):
     new_vessels[1,0:3] = results[5][idx]
     new_vessels[1,3:6] = data[vessel,3:6]
     new_vessels[1,21]  = results[0][idx]*results[3][idx] + radius_buffer
-    new_vessels[1,15:16] = data[vessel,15:16]
+    new_vessels[1,15:17] = data[vessel,15:17]
     new_vessels[1,17]  = np.float(vessel)
     new_vessels[2,0:3] = results[5][idx]
     new_vessels[2,3:6] = terminal
     new_vessels[2,21]  = results[0][idx]*results[2][idx] + radius_buffer
     new_vessels[2,17]  = np.float(vessel)
+    #print(new_vessels[:,-1])
+    vessel_parent = data[vessel,17]
+    vessel_child1 = data[vessel,15]
+    vessel_child2 = data[vessel,16]
+    vessel_sister_i = data[int(vessel_parent),15]
+    vessel_sister_j = data[int(vessel_parent),16]
     for i in range(new_vessels.shape[0]):
         ##Check Self
+        #start = time.time()
         n = sphere_proximity(data,new_vessels[i,:])
+        #print(time.time()-start)
+        if len(np.argwhere(n==vessel)) > 0:
+            n = np.delete(n,np.argwhere(n==vessel).flatten())
+        if len(np.argwhere(n==int(vessel_parent))) > 0:
+            n = np.delete(n,np.argwhere(n==int(vessel_parent)).flatten())
+        if len(np.argwhere(n==int(vessel_child1))) > 0:
+            n = np.delete(n,np.argwhere(n==int(vessel_child1)).flatten())
+        if len(np.argwhere(n==int(vessel_child2))) > 0:
+            n = np.delete(n,np.argwhere(n==int(vessel_child2)).flatten())
+        if len(np.argwhere(n==int(vessel_sister_i))) > 0:
+            n = np.delete(n,np.argwhere(n==int(vessel_sister_i)).flatten())
+        if len(np.argwhere(n==int(vessel_sister_j))) > 0:
+            n = np.delete(n,np.argwhere(n==int(vessel_sister_j)).flatten())
         if len(np.argwhere(n==int(new_vessels[i,-1]))) > 0:
             n = np.delete(n,np.argwhere(n==int(new_vessels[i,-1])).flatten())
+            #print('self')
         #Check Parent
         if len(np.argwhere(n==int(new_vessels[i,17]))) > 0:
             n = np.delete(n,np.argwhere(n==int(new_vessels[i,17])).flatten())
+            #print('parent')
         #Check Children
         if len(np.argwhere(n==int(new_vessels[i,15]))) > 0:
             n = np.delete(n,np.argwhere(n==int(new_vessels[i,15])).flatten())
+            #print('children')
         if len(np.argwhere(n==int(new_vessels[i,16]))) > 0:
             n = np.delete(n,np.argwhere(n==int(new_vessels[i,16])).flatten())
+            #print('children')
         #Check Sister
         if int(new_vessels[i,17]) != -1:
             parent = int(new_vessels[i,17])
@@ -180,6 +295,7 @@ def collision_free(data,results,idx,terminal,vessel,radius_buffer):
                 n = np.delete(n,np.argwhere(n==int(data[parent,15])).flatten())
             if len(np.argwhere(n==int(data[parent,16]))) > 0:
                 n = np.delete(n,np.argwhere(n==int(data[parent,16])).flatten())
+                #print('sister')
         #else:
         #    if len(np.argwhere(n==int(new_vessels[i,-1]))) > 0:
         #        n = np.delete(n,np.argwhere(n==int(new_vessels[i,-1])).flatten())
@@ -188,6 +304,73 @@ def collision_free(data,results,idx,terminal,vessel,radius_buffer):
             continue
         else:
             if obb(data[n,:],new_vessels[i,:]):
+                #print(n)
+                #print(new_vessels[:,-1])
+                """
+                colors = vtk.vtkNamedColors()
+                models = []
+                actors = []
+                background_color='white'
+                resolution=100
+                center = tuple((new_vessels[i,0:3] + new_vessels[i,3:6])/2)
+                radius = new_vessels[i,21]
+                direction = tuple(new_vessels[i,12:15])
+                vessel_length = new_vessels[i,20]
+                cyl = vtk.vtkTubeFilter()
+                line = vtk.vtkLineSource()
+                line.SetPoint1(new_vessels[i,0],new_vessels[i,1],new_vessels[i,2])
+                line.SetPoint2(new_vessels[i,3],new_vessels[i,4],new_vessels[i,5])
+                cyl.SetInputConnection(line.GetOutputPort())
+                cyl.SetRadius(radius)
+                cyl.SetNumberOfSides(resolution)
+                models.append(cyl)
+                mapper = vtk.vtkPolyDataMapper()
+                actor  = vtk.vtkActor()
+                mapper.SetInputConnection(cyl.GetOutputPort())
+                actor.SetMapper(mapper)
+                actor.GetProperty().SetColor(colors.GetColor3d('green'))
+                actors.append(actor)
+                for edge in n:
+                    center = tuple((data[edge,0:3] + data[edge,3:6])/2)
+                    radius = data[edge,21]
+                    direction = tuple(data[edge,12:15])
+                    vessel_length = data[edge,20]
+                    cyl = vtk.vtkTubeFilter()
+                    line = vtk.vtkLineSource()
+                    line.SetPoint1(data[edge,0],data[edge,1],data[edge,2])
+                    line.SetPoint2(data[edge,3],data[edge,4],data[edge,5])
+                    cyl.SetInputConnection(line.GetOutputPort())
+                    cyl.SetRadius(radius)
+                    cyl.SetNumberOfSides(resolution)
+                    models.append(cyl)
+                    mapper = vtk.vtkPolyDataMapper()
+                    actor  = vtk.vtkActor()
+                    mapper.SetInputConnection(cyl.GetOutputPort())
+                    actor.SetMapper(mapper)
+                    if edge in n:
+                        #print('True')
+                        actor.GetProperty().SetColor(colors.GetColor3d('blue'))
+                    if edge == int(new_vessels[i,-1]):
+                        print(edge)
+                        actor.GetProperty().SetColor(colors.GetColor3d('green'))
+                    else:
+                        actor.GetProperty().SetColor(colors.GetColor3d('red'))
+                    actors.append(actor)
+                renderer = vtk.vtkRenderer()
+                renderer.SetBackground(colors.GetColor3d(background_color))
+
+                render_window = vtk.vtkRenderWindow()
+                render_window.AddRenderer(renderer)
+                render_window.SetWindowName('SimVascular Vessel Collision')
+
+                interactor = vtk.vtkRenderWindowInteractor()
+                interactor.SetRenderWindow(render_window)
+                for actor in actors:
+                     renderer.AddActor(actor)
+
+                render_window.Render()
+                interactor.Start()
+                """
                 return False
     return True
 
@@ -236,12 +419,32 @@ def collision_free_fd(data,vessel):
             continue
         else:
             if obb(tree[n,:],new_vessels[i,:]):
+                """
                 colors = vtk.vtkNamedColors()
                 models = []
                 actors = []
                 background_color='white'
                 resolution=100
-                for edge in range(data.shape[0]):
+                for edge in range(new_vessels.shape[0]):
+                    center = tuple((new_vessels[edge,0:3] + new_vessels[edge,3:6])/2)
+                    radius = new_vessels[edge,21]
+                    direction = tuple(new_vessels[edge,12:15])
+                    vessel_length = new_vessels[edge,20]
+                    cyl = vtk.vtkTubeFilter()
+                    line = vtk.vtkLineSource()
+                    line.SetPoint1(new_vessels[edge,0],new_vessels[edge,1],new_vessels[edge,2])
+                    line.SetPoint2(new_vessels[edge,3],new_vessels[edge,4],new_vessels[edge,5])
+                    cyl.SetInputConnection(line.GetOutputPort())
+                    cyl.SetRadius(radius)
+                    cyl.SetNumberOfSides(resolution)
+                    models.append(cyl)
+                    mapper = vtk.vtkPolyDataMapper()
+                    actor  = vtk.vtkActor()
+                    mapper.SetInputConnection(cyl.GetOutputPort())
+                    actor.SetMapper(mapper)
+                    actor.GetProperty().SetColor(colors.GetColor3d('red'))
+                    actors.apend(actor)
+                for edge in n:
                     center = tuple((data[edge,0:3] + data[edge,3:6])/2)
                     radius = data[edge,21]
                     direction = tuple(data[edge,12:15])
@@ -279,6 +482,7 @@ def collision_free_fd(data,vessel):
 
                 render_window.Render()
                 interactor.Start()
+                """
                 return False
     return True
 
