@@ -6,6 +6,7 @@ from mpl_toolkits.mplot3d import Axes3D, proj3d, art3d
 from mpl_toolkits.mplot3d.art3d import Line3DCollection
 import vtk
 from scipy import optimize
+from tqdm import tqdm
 
 from ..branch_addition.basis import tangent_basis
 
@@ -250,15 +251,30 @@ def get_interpolated_sv_data(data):
             frame.extend(normals[idx][jdx])
             frames.append(frame)
         path_frames.append(frames)
-    interps = []
+    interp_xyz = []
+    interp_r   = []
+    interp_n   = []
     for idx in range(len(branches)):
-        if len(path_frames[idx]) == 2:
-            interps.append(splprep(np.array(path_frames[idx]).T,k=1,s=0))
-        elif len(path_frames[idx]) == 3:
-            interps.append(splprep(np.array(path_frames[idx]).T,k=2,s=0))
+        p = np.array(points[idx]).T
+        r = np.array(radii[idx]).T
+        n = np.array(normals[idx]).T
+        if len(points[idx]) == 2:
+            interp_xyz.append(splprep(p,k=1,s=0))
+            r = np.vstack((interp_xyz[-1][1],r))
+            interp_r.append(splprep(r,k=1,s=0))
+            #interp_n.append(splprep(n,k=1,s=0))
+        elif len(points[idx]) == 3:
+            interp_xyz.append(splprep(p,k=2,s=0))
+            r = np.vstack((interp_xyz[-1][1],r))
+            interp_r.append(splprep(r,k=2,s=0))
+            #interp_n.append(splprep(n,k=2,s=0))
         else:
-            interps.append(splprep(np.array(path_frames[idx]).T,s=0))
-    return interps,path_frames,branches
+            interp_xyz.append(splprep(p,s=0))
+            r = np.vstack((interp_xyz[-1][1],r))
+            interp_r.append(splprep(r,s=0))
+            #interp_n.append(splprep(n,s=0))
+
+    return interp_xyz,interp_r,interp_n,path_frames,branches
 
 def get_sv_data(network):
     if isinstance(network,tree):
@@ -277,7 +293,7 @@ def get_sv_data(network):
                 branches.append(get_branches(graft.data))
                 connections.append(network.connections[idx][jdx])
 
-def plot_interp_vessels(interps,normals=True):
+def plot_interp_vessels(interp_xyz,interp_r,normals=True):
     n = np.linspace(0,1,200)
     fig = plt.figure()
     ax1  = fig.add_subplot(111,projection='3d')
@@ -286,12 +302,12 @@ def plot_interp_vessels(interps,normals=True):
     #ax4  = fig.add_subplot(234,projection='3d')
     #ax5  = fig.add_subplot(235,projection='3d')
     #ax6  = fig.add_subplot(236,projection='3d')
-    for idx,b in enumerate(interps):
+    for idx in enumerate(range(interp_xyz)):
         if idx > 0:
             break
-        x,y,z,r,nx,ny,nz = splev(n,b[0])
-        nx,ny,nz,dr,dnx,dny,dnz = splev(n,b[0],der=1)
-        nxx,nyy,nzz,_,_,_,_ = splev(n,b[0],der=2)
+        x,y,z = splev(n,interp_xyz[idx][0])
+        nx,ny,nz = splev(n,interp_xyz[idx][0],der=1)
+        nxx,nyy,nzz = splev(n,inerp_xyz[idx][0],der=2)
         #nxxx,nyyy,nzzz,_,_,_,_ = splev(n,b[0],der=3)
         ax1.plot3D(x,y,z,c='b')
         ax1.scatter3D(x[0],y[0],z[0],c='black')
@@ -311,7 +327,7 @@ def plot_interp_vessels(interps,normals=True):
         ax1.scatter3D(x[idx_yy],y[idx_yy],z[idx_yy],c='g')
         ax1.scatter3D(x[idx_zz],y[idx_zz],z[idx_zz],c='g')
         if b[0][-1] >= 3:
-            nxxx,nyyy,nzzz,_,_,_,_ = splev(n,b[0],der=3)
+            nxxx,nyyy,nzzz = splev(n,interp_xyz[idx][0],der=3)
             idx_xxx = np.argwhere(np.diff(np.sign(nxxx))!=0).flatten()
             idx_yyy = np.argwhere(np.diff(np.sign(nyyy))!=0).flatten()
             idx_zzz = np.argwhere(np.diff(np.sign(nzzz))!=0).flatten()
@@ -339,7 +355,7 @@ def plot_interp_vessels(interps,normals=True):
     plt.show()
 
 from sympy import Point3D,Line3D,Plane,Float
-def contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buffer,contours,t1,t2):
+def contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buffer,contours,t1,t2,n_sides=4):
     """
     d1 = x1*nx1+y1*ny1+z1*nz1
     d2 = x2*nx2+y2*ny2+z2*nz2
@@ -387,8 +403,8 @@ def contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buf
     n = np.linspace(t1,t2)
     length = ((x2-x1)**2+(y2-y1)**2+(z2-z1)**2)**(1/2)
     if radius_buffer is not None:
-        r1 += r1 + r1*radius_buffer
-        r2 += r2 + r2*radius_buffer
+        r1 += r1*radius_buffer
+        r2 += r2*radius_buffer
     contour_1 = vtk.vtkRegularPolygonSource()
     contour_2 = vtk.vtkRegularPolygonSource()
     contour_1.SetCenter([x1,y1,z1])
@@ -397,11 +413,15 @@ def contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buf
     contour_2.SetRadius(r2)
     contour_1.SetNormal([nx1,ny1,nz1])
     contour_2.SetNormal([nx2,ny2,nz2])
-    contour_1.SetNumberOfSides(4)
-    contour_2.SetNumberOfSides(4)
+    contour_1.SetNumberOfSides(n_sides)
+    contour_2.SetNumberOfSides(n_sides)
+    contour_11 = vtk.vtkContourTriangulator()
+    contour_22 = vtk.vtkContourTriangulator()
+    contour_11.SetInputConnection(contour_1.GetOutputPort())
+    contour_22.SetInputConnection(contour_2.GetOutputPort())
     collision = vtk.vtkCollisionDetectionFilter()
-    collision.SetInputConnection(0,contour_1.GetOutputPort())
-    collision.SetInputConnection(1,contour_2.GetOutputPort())
+    collision.SetInputConnection(0,contour_11.GetOutputPort())
+    collision.SetInputConnection(1,contour_22.GetOutputPort())
     transform = vtk.vtkTransform()
     matrix = vtk.vtkMatrix4x4()
     collision.SetTransform(0,transform)
@@ -410,6 +430,35 @@ def contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buf
     collision.GenerateScalarsOn()
     collision.Update()
     if collision.GetNumberOfContacts() > 0:
+        """
+        actors = []
+        colors = vtk.vtkNamedColors()
+        mapper_1 = vtk.vtkPolyDataMapper()
+        mapper_2 = vtk.vtkPolyDataMapper()
+        actor_1 = vtk.vtkActor()
+        actor_2 = vtk.vtkActor()
+        mapper_1.SetInputConnection(contour_11.GetOutputPort())
+        mapper_2.SetInputConnection(contour_22.GetOutputPort())
+        actor_1.SetMapper(mapper_1)
+        actor_2.SetMapper(mapper_2)
+        actor_1.GetProperty().SetColor(colors.GetColor3d('blue'))
+        actor_2.GetProperty().SetColor(colors.GetColor3d('blue'))
+        actors.append(actor_1)
+        actors.append(actor_2)
+        renderer = vtk.vtkRenderer()
+        renderer.SetBackground(colors.GetColor3d('white'))
+
+        render_window = vtk.vtkRenderWindow()
+        render_window.AddRenderer(renderer)
+        render_window.SetWindowName('Pathline with Contours')
+
+        interactor = vtk.vtkRenderWindowInteractor()
+        interactor.SetRenderWindow(render_window)
+        for actor in actors:
+            renderer.AddActor(actor)
+        render_window.Render()
+        interactor.Start()
+        """
         return True
     else:
         return False
@@ -437,49 +486,56 @@ def contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buf
     else:
         return False
     """
-def contour_check(contours,t0,t1,radius_buffer):
+def contour_check(interp_xyz,interp_r,t0,t1,radius_buffer,n_sides=4):
     tl  = (t1 - t0)/10
-    x1,y1,z1,r1,_,_,_   = splev(t0,contours[0])
-    x11,y11,z11,_,_,_,_   = splev(t0+tl,contours[0])
-    nx1 = x11-x1
-    ny1 = y11-y1
-    nz1 = z11-z1
-    #nx1,ny1,nz1,_,_,_,_ = splev(t0,contours[0])
+    x1,y1,z1      = splev(t0,interp_xyz[0])
+    _,r1            = splev(t0,interp_r[0])
+    #x11,y11,z11,_,_,_,_   = splev(t0+tl,contours[0])
+    #nx1 = x11-x1
+    #ny1 = y11-y1
+    #nz1 = z11-z1
+    nx1,ny1,nz1 = splev(t0,interp_xyz[0],der=1)
     n1  = np.array([nx1,ny1,nz1])
-    #mag = np.linalg.norm(n1,axis=0)
+    mag = np.linalg.norm(n1,axis=0)
+    n1 = n1/mag
     #nx1 = -mag/nx1
     #ny1 = -mag/ny1
     #nz1 = -mag/nz1
-    x2,y2,z2,r2,_,_,_   = splev(t1,contours[0])
-    x22,y22,z22,_,_,_,_   = splev(t1+tl,contours[0])
-    nx2 = x22-x2
-    ny2 = y22-y2
-    nz2 = z22-z2
-    #nx2,ny2,nz2,_,_,_,_ = splev(t1,contours[0])
+    x2,y2,z2            = splev(t1,interp_xyz[0])
+    _,r2                  = splev(t1,interp_r[0])
+    #x22,y22,z22,_,_,_,_   = splev(t1+tl,contours[0])
+    #nx2 = x22-x2
+    #ny2 = y22-y2
+    #nz2 = z22-z2
+    nx2,ny2,nz2 = splev(t1,interp_xyz[0],der=1)
     n2  = np.array([nx2,ny2,nz2])
-    #mag = np.linalg.norm(n2,axis=0)
+    mag = np.linalg.norm(n2,axis=0)
+    n2 = n2/mag
     #nx2 = -mag/nx2
     #ny2 = -mag/ny2
     #nz2 = -mag/nz2
-    if np.sum(np.isclose(n1-n2,0)) > 1:
-        return False
-    elif np.isclose(nx1,0) and np.isclose(ny1,0):
-        collision = contour_collision(x2,y2,z2,nx2,ny2,nz2,r2,x1,y1,z1,nx1,ny1,nz1,r1,radius_buffer,contours,t0,t1)
-        #print(collision)
-        return collision
-    else:
-        collision = contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buffer,contours,t0,t1)
-        #print(collision)
-        return collision
+    #if np.sum(np.isclose(n1-n2,0)) == 3:
+    #    return False
+    #elif np.isclose(nx1,0) and np.isclose(ny1,0):
+    #print('{} {} {}'.format(x1,y1,z1))
+    #print('{} {} {}'.format(x2,y2,z2))
+    #print()
+    collision = contour_collision(x2,y2,z2,nx2,ny2,nz2,r2,x1,y1,z1,nx1,ny1,nz1,r1,radius_buffer,interp_xyz,t0,t1,n_sides=n_sides)
+    #print(collision)
+    return collision
+    #else:
+    #    collision = contour_collision(x1,y1,z1,nx1,ny1,nz1,r1,x2,y2,z2,nx2,ny2,nz2,r2,radius_buffer,contours,t0,t1,n_sides=n_sides)
+    #    #print(collision)
+    #    return collision
 
-def contour_check_all(interps,radius_buffer):
+def contour_check_all(interp_xyz,interp_r,radius_buffer):
     sample_t = []
     n = np.linspace(0,1,200)
     total_collisions = 0
-    for contours in interps:
+    for kdx in tqdm(range(len(interp_xyz)),desc='Resolving Overlaps'):
         tmp_t  = []
         t_list = []
-        t_list = np.linspace(0,1,4*len(contours[1]))
+        t_list = np.linspace(0,1,4*len(interp_xyz[kdx][1]))
         collision_contours = set([])
         collision_free = set([])
         checking = set(list(range(1,len(t_list)-1)))
@@ -487,10 +543,11 @@ def contour_check_all(interps,radius_buffer):
         for idx in checking:
             collide = False
             for jdx in checking.difference(set([idx])):
-                if contour_check(contours,t_list[idx],t_list[jdx],radius_buffer):
+                if contour_check(interp_xyz[kdx],interp_r[kdx],t_list[idx],t_list[jdx],radius_buffer,n_sides=20):
                     collision_contours.add(idx)
                     collision_contours.add(jdx)
                     collide = True
+                    break
             if not collide:
                 collision_free.add(idx)
             #checking = checking.difference(collision_contours)
@@ -499,22 +556,36 @@ def contour_check_all(interps,radius_buffer):
         collision_free.sort()
         if len(collision_contours) > 0:
             pass
-            #plot_path_contours(contours,t_list,highlight=list(collision_contours))
+            plot_path_contours(interp_xyz[kdx],interp_r[kdx],t_list,highlight=list(collision_contours))
         tmp_t.append(t_list[0])
         for idx in range(len(collision_free)-1):
-            tmp_t.append(t_list[collision_free[idx]])
+
             if collision_free[idx+1] - collision_free[idx]>1:
-                t_op,f_op = optimize_contour(t_list[collision_free[idx]],t_list[collision_free[idx+1]],contours,radius_buffer)
-                #if not contour_check(contours,t_list[collision_free[idx]],t_op,radius_buffer):
-                #    if not contour_check(contours,t_list[collision_free[idx+1]],t_op,radius_buffer):
-                #        tmp_t.append(t_op)
-                if f_op < 0:
-                    tmp_t.append(t_op)
+                tmp_tmp_t = [t_list[collision_free[idx]],t_list[collision_free[idx+1]]]
+                #nx,ny,nz,_,_,_,_ = splev(tmp_tmp_t,contours,der=1)
+                #normals = np.array([nx,ny,nz]).T
+                #tmp_tmp_angles = [np.dot(normals[i,:],normals[i+1,:]) for i in range(normals.shape[0]-1)]
+                for jdx in range(collision_free[idx+1] - collision_free[idx]):
+                    #max_diff = np.argmax(np.diff(tmp_tmp_angles))
+                    nx,ny,nz = splev(tmp_tmp_t,interp_xyz[kdx][0],der=1)
+                    normals = np.array([nx,ny,nz]).T
+                    tmp_tmp_angles = [np.dot(normals[i,:],normals[i+1,:]) for i in range(normals.shape[0]-1)]
+                    max_diff = np.argmin(tmp_tmp_angles)
+                    t_op,f_op = optimize_contour(tmp_tmp_t[max_diff],tmp_tmp_t[max_diff+1],interp_xyz[kdx],interp_r[kdx],radius_buffer,n_sides=20)
+                    if f_op < 0:
+                        tmp_tmp_t.append(t_op)
+                        tmp_tmp_t.sort()
+                    else:
+                        break
+                tmp_tmp_t.pop(-1)
+                tmp_t.extend(tmp_tmp_t)
+            else:
+                tmp_t.append(t_list[collision_free[idx]])
         tmp_t.append(t_list[collision_free[-1]])
         tmp_t.append(t_list[-1])
         if len(collision_contours) > 0:
             pass
-            #plot_path_contours(contours,tmp_t)
+            plot_path_contours(interp_xyz[kdx],interp_r[kdx],tmp_t)
         if len(tmp_t) == 1:
             tmp_t.append(1)
         else:
@@ -523,16 +594,17 @@ def contour_check_all(interps,radius_buffer):
     print('Total Collisions Resolved: '+str(total_collisions))
     return sample_t
 
-def sv_data(interps,radius_buffer=0):
+def sv_data(interp_xyz,interp_r,radius_buffer=0.25):
     points  = []
     radii   = []
     normals = []
-    sample_t = contour_check_all(interps,radius_buffer)
+    sample_t = contour_check_all(interp_xyz,interp_r,radius_buffer)
     fig = plt.figure()
     ax = fig.add_subplot(111,projection='3d')
-    for idx,tck in enumerate(interps):
-        x,y,z,r,_,_,_    = splev(np.array(sample_t[idx]),tck[0])
-        nx,ny,nz,_,_,_,_ = splev(np.array(sample_t[idx]),tck[0],der=1)
+    for idx in range(len(interp_xyz)):
+        x,y,z    = splev(np.array(sample_t[idx]),interp_xyz[idx][0])
+        _,r        = splev(np.array(sample_t[idx]),interp_r[idx][0])
+        nx,ny,nz = splev(np.array(sample_t[idx]),interp_xyz[idx][0],der=1)
         l  = np.linalg.norm(np.array([nx,ny,nz]),axis=0)
         nx = nx/l
         ny = ny/l
@@ -577,7 +649,7 @@ def plot_branches(data,branches):
     plt.legend()
     plt.show()
 
-def plot_path_contours(interp,t_list,highlight=[]):
+def plot_path_contours(interp_xyz,interp_r,t_list,highlight=[]):
     disks = []
     lines = []
     points = vtk.vtkPoints()
@@ -587,8 +659,9 @@ def plot_path_contours(interp,t_list,highlight=[]):
     actors = []
     colors = vtk.vtkNamedColors()
     for t in t_list:
-        x,y,z,r,_,_,_ = splev(t,interp[0])
-        nx,ny,nz,_,_,_,_ = splev(t,interp[0],der=1)
+        x,y,z = splev(t,interp_xyz[0])
+        _,r     = splev(t,interp_r[0])
+        nx,ny,nz = splev(t,interp_xyz[0],der=1)
         points.InsertNextPoint([x,y,z])
         radii.append(r)
         normals.append([nx,ny,nz])
@@ -638,25 +711,26 @@ def plot_path_contours(interp,t_list,highlight=[]):
     render_window.Render()
     interactor.Start()
 
-def optimize_contour(t0,t1,interp,radius_buffer):
-    nx0,ny0,nz0,_,_,_,_ = splev(t0,interp[0],der=1)
-    nx1,ny1,nz1,_,_,_,_ = splev(t1,interp[0],der=1)
+def optimize_contour(t0,t1,interp_xyz,interp_r,radius_buffer,t_buffer=0.1,n_sides=4):
+    nx0,ny0,nz0 = splev(t0,interp_xyz[0],der=1)
+    nx1,ny1,nz1 = splev(t1,interp_xyz[0],der=1)
     n0 = np.array([nx0,ny0,nz0])
     n1 = np.array([nx1,ny1,nz1])
     n0 = n0/np.linalg.norm(n0) #.reshape(-1,1)
     n1 = n1/np.linalg.norm(n1) #.reshape(-1,1)
-    def func(t,interp=interp,n0=n0,n1=n1,t0=t0,t1=t1):
-        nx,ny,nz,_,_,_,_ = splev(t,interp[0],der=1)
+    t_bound_buff = (t1-t0)*t_buffer
+    def func(t,interp_xyz=interp_xyz,interp_r=interp_r,n0=n0,n1=n1,t0=t0,t1=t1):
+        nx,ny,nz = splev(t,interp_xyz[0],der=1)
         n = np.array([nx,ny,nz])
         n = n/np.linalg.norm(n) #.reshape(-1,1)
         #print(-np.dot(n0,n)*np.dot(n1,n))
         a = np.dot(n0,n)
         b = np.dot(n1,n)
         c = 0
-        if contour_check(interp,t0,t,radius_buffer=radius_buffer):
+        if contour_check(interp_xyz,interp_r,t0,t,radius_buffer=radius_buffer,n_sides=n_sides):
             c = 100
-        if contour_check(interp,t1,t,radius_buffer=radius_buffer):
+        if contour_check(interp_xyz,interp_r,t1,t,radius_buffer=radius_buffer,n_sides=n_sides):
             c = 100
-        return -(a*b + a + b) + c
-    result = optimize.minimize_scalar(func,bounds=(t0,t1),method='bounded')
+        return -(a*b+(1-abs(a-b))) + c
+    result = optimize.minimize_scalar(func,bounds=(t0+t_bound_buff,t1-t_bound_buff),method='bounded')
     return result.x,result.fun
