@@ -5,17 +5,20 @@ presolver="""if not terminating:
     resistance = dict([])
     mass = vtk.vtkMassProperties()
     max_surface_area = 0
+    total_res = 0
     for i,face in enumerate(mesher.get_model_face_ids()):
         if face == walls[0]:
             continue
         f_poly = mesher.get_face_polydata(face)
         mass.SetInputData(f_poly)
         resistance[face] = mass.GetSurfaceArea()
+        total_res += 1/resistance[face]
         if resistance[face] > max_surface_area:
             skip = i + 1
             inlet = face
             max_surface_area = resistance[face]
         svpre_construction += 'set_surface_id_vtp mesh-complete/mesh-surfaces/cap_{{}}.vtp {{}}\\n'.format(face,i+1)
+    total_res -= 1/max_surface_area
     svpre_construction += 'fluid_density {}\\n'
     svpre_construction += 'fluid_viscosity {}\\n'
     svpre_construction += 'initial_pressure {}\\n'
