@@ -19,30 +19,31 @@ from copy import deepcopy
 from itertools import combinations
 from tqdm import tqdm
 import pickle
+import matplotlib.pyplot as plt
 
 class tree:
     """
-    # Data structure:
-    # index: 0:2   -> proximal node coordinates
-    # index: 3:5   -> distal node coordinates
-    # index: 6:8   -> unit basis U (all)
-    # index: 9:11  -> unit basis V (all)
-    # index: 12:14 -> unit basis W (axial direction) (all)
-    # index: 15,16 -> children (-1 means no child)
-    # index: 17    -> parent (NA)
-    # index: 18    -> proximal node index (only real edges)
-    # index: 19    -> distal node index (only real edges)
-    # index: 20    -> length (path length)
-    # index: 21    -> radius (what are the units?)
-    # index: 22    -> flow (NA)
-    # index: 23    -> left bifurcation (NA)
-    # index: 24    -> right bifurcation (NA)
-    # index: 25    -> reduced resistance (NA)
-    # index: 26    -> depth (NA)
-    # index: 27    -> reduced downstream length (NA)
-    # index: 28    -> root radius scaling factor (same as edge for intermediate)
-    # index: 29    -> edge that subedge belongs to (if actual edge; self pointer)
-    # index: 30    -> self identifying index (-1 if intermediate)
+    Data structure:\n
+    index: 0:2   -> proximal node coordinates \n
+    index: 3:5   -> distal node coordinates \n
+    index: 6:8   -> unit basis U (all) \n
+    index: 9:11  -> unit basis V (all) \n
+    index: 12:14 -> unit basis W (axial direction) (all) \n
+    index: 15,16 -> children (-1 means no child) \n
+    index: 17    -> parent (NA) \n
+    index: 18    -> proximal node index (only real edges) \n
+    index: 19    -> distal node index (only real edges) \n
+    index: 20    -> length (path length) \n
+    index: 21    -> radius (what are the units?) \n
+    index: 22    -> flow (NA) \n
+    index: 23    -> left bifurcation (NA) \n
+    index: 24    -> right bifurcation (NA) \n
+    index: 25    -> reduced resistance (NA) \n
+    index: 26    -> depth (NA) \n
+    index: 27    -> reduced downstream length (NA) \n
+    index: 28    -> root radius scaling factor (same as edge for intermediate) \n
+    index: 29    -> edge that subedge belongs to (if actual edge; self pointer) \n
+    index: 30    -> self identifying index (-1 if intermediate) \n
     """
     def __init__(self):
         #self.parameters = {'gamma'   : 3.0,
@@ -55,9 +56,25 @@ class tree:
         #                   'edge_num': 0}
         self.set_parameters()
         self.data = np.zeros((1,31))
-        self.radius_buffer = 0.1
+        self.radius_buffer = 0.01
         self.fraction = None
         self.set_assumptions()
+        self.rng_points = []
+        self.time = {'search':[],
+                     'constraints':[],
+                     'local_optimize':[],
+                     'collision':[],
+                     'close_time':[],
+                     'search_1':[],
+                     'search_2':[],
+                     'search_3':[],
+                     'search_4':[],
+                     'add_time':[],
+                     'add_1':[],
+                     'add_2':[],
+                     'add_3':[],
+                     'add_4':[],
+                     'total':[]}
 
     def set_parameters(self,**kwargs):
         self.parameters = {}
@@ -77,6 +94,7 @@ class tree:
         self.hollow      = kwargs.get('hollow',False)
         self.dimension   = kwargs.get('dimension',3)
         self.clamped_root= kwargs.get('clamped_root',False)
+        self.nonconvex_counter = 0
 
     def show_assumptions(self):
         print('homogeneous : {}'.format(self.homogeneous))
@@ -91,7 +109,7 @@ class tree:
         self.fraction = (self.boundary.volume**(1/3))/20
 
     def set_root(self,low=-1,high=0,start=None,direction=None,
-                 limit_high=None,limit_low=None):
+                 limit_high=None,limit_low=None,niter=200):
         Qterm = self.parameters['Qterm']
         gamma = self.parameters['gamma']
         nu    = self.parameters['nu']
@@ -100,19 +118,20 @@ class tree:
         result = set_root(self.data,self.boundary,Qterm,
                           gamma,nu,Pperm,Pterm,self.fraction,
                           self.homogeneous,self.convex,self.directed,
-                          start,direction,limit_high,limit_low,low=-1,high=0)
+                          start,direction,limit_high,limit_low,self.boundary.volume,
+                          low=-1,high=0,niter=niter)
         self.data = result[0]
         self.sub_division_map = result[1]
         self.sub_division_index = result[2]
         self.parameters['edge_num'] = 1
-        self.sub_division_map = [-1]
-        self.sub_division_index = np.array([0])
+        #self.sub_division_map = [-1]
+        #self.sub_division_index = np.array([0])
 
     def add(self,low,high,isforest=False):
-        vessel,data,sub_division_map,sub_division_index = add_branch(self,low,high,threshold_exponent=1.5,
+        vessel,data,sub_division_map,sub_division_index = add_branch(self,low,high,threshold_exponent=0.5,
                                                                      threshold_adjuster=0.75,all_max_attempts=40,
-                                                                     max_attemps=10,sampling=20,max_skip=8,
-                                                                     flow_ratio=8)
+                                                                     max_attemps=3,sampling=20,max_skip=8,
+                                                                     flow_ratio=None)
         if isforest:
             return vessel,data,sub_division_map,sub_division_index
         else:
@@ -122,11 +141,34 @@ class tree:
             self.sub_division_index = sub_division_index
 
     def n_add(self,n):
-        for i in tqdm(range(n),desc='Adding vessels'):
+        self.rng_points,_ = self.boundary.pick(size=40*n,homogeneous=True)
+        self.rng_points = self.rng_points.tolist()
+        #for i in tqdm(range(n),desc='Adding vessels'):
+        for i in range(n):
+            #self.rng_points = self.rng_points.tolist()
             self.add(-1,0)
 
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['search'][1:],label='search time')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['constraints'][1:],label='constraint time')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['local_optimize'][1:],label='local time')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['collision'][1:],label='collision time')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['search_1'][1:],label='search 1 time')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['search_2'][1:],label='search 2 time')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['search_3'][1:],label='search 3 time')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['search_4'][1:],label='search 4 time')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['close_time'][1:],label='close_time time')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['add_time'][1:],label='add_time')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['add_1'][1:],label='add_1')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['add_2'][1:],label='add_2')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['add_3'][1:],label='add_3')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['add_4'][1:],label='add_4')
+        #plt.plot(list(range(len(self.time['search'][1:]))),self.time['total'][1:],label='total')
+        #plt.ylim([0,0.1])
+        #plt.legend()
+        #plt.show()
+
     def show(self,surface=False,vessel_colors='red',background_color='white',
-             resolution=100,show_segments=True):
+             resolution=100,show_segments=True,save=False,name=None,show=True):
         models = []
         actors = []
         colors = vtk.vtkNamedColors()
@@ -159,9 +201,17 @@ class tree:
                 actor  = vtk.vtkActor()
                 mapper.SetInputData(self.boundary.polydata)
                 actor.SetMapper(mapper)
-                actor.GetProperty().SetColor(colors.GetColor3d('blue'))
-                actor.GetProperty().SetOpacity(0.5)
+                actor.GetProperty().SetColor(colors.GetColor3d('red'))
+                actor.GetProperty().SetOpacity(0.25)
                 actors.append(actor)
+                for other_surface in self.boundary.subtracted_volumes:
+                    mapper = vtk.vtkPolyDataMapper()
+                    actor  = vtk.vtkActor()
+                    mapper.SetInputData(other_surface.polydata)
+                    actor.SetMapper(mapper)
+                    actor.GetProperty().SetColor(colors.GetColor3d('blue'))
+                    actor.GetProperty().SetOpacity(0.5)
+                    actors.append(actor)
             renderer = vtk.vtkRenderer()
             renderer.SetBackground(colors.GetColor3d(background_color))
 
@@ -175,8 +225,26 @@ class tree:
             for actor in actors:
                 renderer.AddActor(actor)
 
-            render_window.Render()
-            interactor.Start()
+            if save:
+                render_window.Render()
+                w2if = vtk.vtkWindowToImageFilter()
+                w2if.SetInput(render_window)
+                w2if.Update()
+
+                writer = vtk.vtkPNGWriter()
+                if name is None:
+                    tag = time.gmtime()
+                    name = str(tag.tm_year)+str(tag.tm_mon)+str(tag.tm_mday)+str(tag.tm_hour)+str(tag.tm_min)+str(tag.tm_sec)
+                writer.SetFileName(os.getcwd()+os.sep+'{}.png'.format(name))
+                writer.SetInputData(w2if.GetOutput())
+                writer.Write()
+            elif show==True:
+                render_window.Render()
+                interactor.Start()
+            elif show==False:
+                for m in models:
+                    m.Update()
+            return models
 
     def save(self,filename=None):
         if filename is None:
@@ -211,8 +279,8 @@ class tree:
         file.close()
 
     def export(self,steady=True,apply_distal_resistance=True,gui=True):
-        interps,frames,branches = get_interpolated_sv_data(self.data)
-        points,radii,normals    = sv_data(interps,radius_buffer=self.radius_buffer)
+        interp_xyz,interp_r,interp_n,frames,branches = get_interpolated_sv_data(self.data)
+        points,radii,normals    = sv_data(interp_xyz,interp_r,radius_buffer=self.radius_buffer)
         if steady:
             time = [0, 1]
             flow = [self.data[0,22], self.data[0,22]]
@@ -235,11 +303,12 @@ class forest:
     def __init__(self,boundary=None,number_of_networks=1,
                  trees_per_network=[2],scale=None,start_points=None,
                  directions=None,root_lengths_high=None,
-                 root_lengths_low=None):
+                 root_lengths_low=None,convex=False):
         self.networks    = []
         self.connections = []
         self.backup      = []
         self.boundary    = boundary
+        self.convex      = convex
         self.number_of_networks = number_of_networks
         self.trees_per_network = trees_per_network
         if isinstance(start_points,type(None)):
@@ -269,6 +338,7 @@ class forest:
                       " with length equal to number_of_networks")
             for jdx in range(self.trees_per_network[idx]):
                 tmp = tree()
+                tmp.set_assumptions(convex=self.convex)
                 if self.directions[idx][jdx] is not None:
                     tmp.clamped_root = True
                 tmp.set_boundary(self.boundary)
@@ -375,7 +445,7 @@ class forest:
         if show:
             plot = pv.Plotter()
             colors = ['r','b','g','y']
-            plot.set_background(color=[253,250,219])
+            #plot.set_background(color=[253,250,219])
             for model_network in model_networks:
                 for color_idx, model_tree in enumerate(model_network):
                     for model in model_tree:
@@ -392,3 +462,38 @@ class forest:
             #plot.orbit_on_path(path,write_frames=True)
             #plot.close()
             return plot
+
+def perfusion_territory(tree,subdivisions=0,mesh_file=None,tree_file=None):
+    terminals = tree.data[tree.data[:,15]<0,:]
+    terminals = terminals[terminals[:,16]<0,:]
+    territory_id = []
+    territory_volumes = np.zeros(len(terminals))
+    vol = tree.boundary.tet.grid.compute_cell_sizes().cell_data['Volume']
+    #for idx, cell_center in tqdm(enumerate(tree.boundary.tet.grid.cell_centers().points),desc='Calculating Perfusion Territories'):
+    for idx, cell_center in enumerate(tree.boundary.tet.grid.cell_centers().points):
+        closest_terminal = np.argmin(np.linalg.norm(cell_center.reshape(1,-1)-terminals[:,3:6],axis=1))
+        territory_id.append(closest_terminal)
+        territory_volumes[closest_terminal] += vol[idx]
+    territory_id = np.array(territory_id)
+    tree.boundary.tet.grid['perfusion_territory_id'] = territory_id
+    if mesh_file is not None:
+        tree.boundary.tet.grid.save(mesh_file)
+    else:
+        tree.boundary.tet.grid.save('example.vtu')
+    models = tree.show(show=False)
+    append = vtk.vtkAppendPolyData()
+    #append.UserManagedInputsOn()
+    #append.SetInputDataObject(models[0])
+    #append.SetNumberOfInputs(len(models))
+    for i in range(len(models)):
+        append.AddInputData(models[i].GetOutput())
+    append.Update()
+    total_tree = append.GetOutput()
+    writer = vtk.vtkXMLPolyDataWriter()
+    if tree_file is not None:
+        writer.SetFileName(tree_file)
+    else:
+        writer.SetFileName(os.getcwd()+os.sep+'example_tree.vtp')
+    writer.SetInputDataObject(total_tree)
+    writer.Update()
+    return territory_id,territory_volumes/np.sum(territory_volumes)
